require 'securerandom'
require 'pp'

require 'wildsight/agent/tools'
require 'wildsight/agent/config'
require 'wildsight/agent/logger'

module Wildsight
  module Agent

    def self.default
      find
    end

    def self.find(name = 'default')
      @agents ||= {}
      @agents[name] ||= Wildsight::Agent::Agent.new
    end

    class Agent

      include Wildsight::Tools

      def initialize(config = (Wildsight::Agent::Config.load))
        @config = config
        setup
      end

      def logger
        @logger
      end

      def context(name = nil)
        name ||= SecureRandom.uuid
        name = name.to_s
        unless @contexts[name]
          @contexts[name] = Wildsight::Context::Context.new(name, self)
          self.logger.log(:debug) { "New context registered #{name}"}
        end
        return @contexts[name]
      end

      def unknown_context
        context('unknown_context')
      end

      def unregister_context(context)
        context = context.name if context.kind_of?(Wildsight::Context::Context)
        name = context.to_s
        self.logger.log(:debug) { "Context unregistered #{name}"}
        @contexts.delete(name)
      end

      def submit(type, event)
        (@events[type] ||= []) << event
        if @events.size > @config['agent']['threshold']
          self.logger.log(:debug) { 'Uploading collected data' }
          upload
        end
      end

      def upload
        payload, @events = @events, {}
        @transport.send(payload) if payload != nil && payload.size > 0
      end

      private

      def setup
        @logger = Wildsight::Agent::Logger.new(@config)
        @logger.log(:debug) { 'Logger initialized' }

        @events = {}
        @contexts = {}

        @transport = Wildsight::Transport.detect.new(self, @config)
        thread
      end

      def thread
        @thread ||= Thread.new {
          while true
            begin
              upload
              sleep(@config['agent']['timeout'])
            rescue Exception => e
              puts e.message
              puts e.backtrace
            end
          end
        }.run
      end

    end

  end
end