require 'logger'

module Wildsight
  module Agent

    class Logger

      TABLE = {
          :debug => ::Logger::DEBUG,
          :info => ::Logger::INFO,
          :warn => ::Logger::WARN,
          :error => ::Logger::ERROR,
          :fatal => ::Logger::FATAL,
          'debug' => ::Logger::DEBUG,
          'info' => ::Logger::INFO,
          'warn' => ::Logger::WARN,
          'error' => ::Logger::ERROR,
          'fatal' => ::Logger::FATAL
      }

      def initialize(config)
        @config = config
        @loggers = []

        if @config['logger']['stdout']
          @loggers << ::Logger.new(STDOUT)
        end

        if @config['logger']['stderr']
          @loggers << ::Logger.new(STDERR)
        end

        if @config['logger']['file'] && @config['logger']['file'].kind_of?(String)
          @loggers << ::Logger.new(@config['logger']['file'])
        end

        @loggers.each { |logger| logger.level = TABLE[@config['logger']['level']] }
      end

      def log(severity, message = nil, &block)
        @loggers.each { |logger| logger.add(TABLE[severity] || ::Logger::UNKNOWN, nil, message, &block) }
      end

    end

  end
end