module Wildsight
  module Context

    def self.detect(thread = Thread.current)
      thread.thread_variable_get('wildsight.context') || Wildsight::Agent.default.unknown_context
    end

    class Context

      def initialize(name, agent)
        @name = name
        @agent = agent
        @data = {}
      end

      def name
        @name
      end

      def data
        @data
      end

      def unregister
        @agent.unregister_context(self)
      end

      def bind_thread(thread = Thread.current)
        thread.thread_variable_set('wildsight.context', self)
      end

      def release_thread(thread = Thread.current)
        thread.thread_variable_set('wildsight.context', nil)
      end

      def event(series, data, values = {}, occurred = DateTime.now, source = Wildsight.source, host = Wildsight.host)
        @agent.submit(:events, {
                          host: host,
                          source: source,
                          series: series,
                          context: @name,
                          occurred: occurred,
                          data: data,
                          values: values
                      })
      end

      def profiler
        @profiler ||= ::Wildsight::Profiler::Profiler.new(self)
      end

      def self.thread_context_id(thread=Thread.current)
        thread.thread_variable_get('wildsight.context')
      end

    end

  end
end