module Wildsight
  module Profiler
    class Profiler

      def initialize(context)
        @context = context
        @data = {}
        @profile = nil
      end

      def duration(name, time = nil, options = {}, &block)
        p = @profile = Profile.new(name, @profile)
        @profile.options = options

        if block_given?
          @profile.profile!(&block)
        else
          @profile.set(time[0], time[1])
        end

        @profile = @profile.close

        durations(name)[:intervals] << p.export
        durations(name)[:duration] += p.duration

        raise p.exception if p.exception
        return p.result
      end

      def data
        @data
      end

      def durations(name)
        @data[name] ||= { intervals: [], duration: 0.0 }
      end

    end

    class Profile

      attr_accessor :options

      def initialize(name, parent)
        @name = name
        @parent = parent
        @subtimes = []
        @result = nil
        @exception = nil
      end

      def set(before, after)
        @before = before
        @after = after
      end

      def profile!(&block)
        @before = DateTime.now.to_f
        begin
          @result = block.call
        rescue Exception => e
          @exception = e
        end
        @after = DateTime.now.to_f
      end

      def close
        @parent << raw if @parent
        @parent
      end

      def result
        @result
      end

      def exception
        @exception
      end

      def <<(time)
        @subtimes << time
      end

      def subtimes
        @subtimes.inject(0.0) { |c,x| c + x }
      end

      def duration
        return raw if options[:raw]
        return raw - subtimes
      end

      def raw
        (@after - @before) * 1000
      end

      def export
        { before: @before, after: @after, duration: duration, raw: raw }
      end

    end

  end
end