module Wildsight
  module Rails

    def self.instrument
      return if @instrumented

      @instrumented = true

      ::AbstractController::Base.class_eval do

        include Wildsight::Rails::ActionController

        alias_method :process_action_without_ws, :process_action

        def wildsight_context
          @_wildsight_context ||= Wildsight::Rack.detect_context(self.respond_to?(:request) ? request.env : {})
        end

        def process_action(*args)
          context = Wildsight::Rack.detect_context(self.respond_to?(:request) ? request.env : {})
          return process_action_without_ws(*args) unless context

          if respond_to?(:request)
            context.data[:rack][:request][:target] = self.class.name + '#' + request.params['action']

            context.data[:rack][:request][:method] = request.method
            context.data[:rack][:request][:path] = request.original_fullpath
            context.data[:rack][:request][:params] = request.filtered_parameters

            context.data[:rack][:request][:session] = {id: request.session.id}

            context.data[:rack][:request][:url] = request.original_url

            context.data[:rack][:request][:remote] = request.remote_ip
          end
          result = context.profiler.duration(:action) { process_action_without_ws(*args) }
          return result
        rescue Exception => e
          raise e
        end

      end

      ::AbstractController::Rendering.class_eval do

        alias_method :render_without_ws, :render

        def render(*args, &block)
          context = Wildsight::Rack.detect_context(self.respond_to?(:request) ? request.env : {})
          result = context.profiler.duration(:view) { render_without_ws(*args, &block) }
          return result
        end

      end

    end

    class Railtie < ::Rails::Railtie

      initializer('wildsight.rails.middleware.top') do |app|
        app.middleware.unshift(Wildsight::Rack::TopMiddleware)
      end

      initializer('wildsight.rails.middleware.bottom') do |app|
        app.middleware.use(Wildsight::Rack::BottomMiddleware)
      end

      initializer('wildsight.rails.logger') do |app|
        ::Rails.logger.extend(ActiveSupport::Logger.broadcast(Wildsight::Context::Logger.new))
      end

      initializer('wildsight.action_controller.extensions') do |app|
        ActiveSupport.on_load(:action_controller) do
          Wildsight::Rails.instrument
        end

      end

      initializer('wildsight.action_controller.metrics') do |app|
        ActiveSupport::Notifications.subscribe('process_action.action_controller') do |*args|
        end
      end

      initializer('wildsight.active_record.extensions') do |app|
        ActiveSupport.on_load(:active_record) do
          include Wildsight::Rails::ActiveRecord
        end
      end

      initializer('wildsight.active_record.metrics') do |app|
        ActiveSupport::Notifications.subscribe('sql.active_record') do |*args|
          context = Wildsight::Context.detect
          if context
            context.profiler.duration(:database, [args[1].to_f, args[2].to_f])

            event = {
                :name => args[4][:name],
                :query => args[4][:sql],
                :connection => args[4][:connection_id].to_s,
            }
            context.event(:active_record, event, {duration: (args[2] - args[1]) * 1000})
          end
        end

      end

    end
  end
end